%% Data fusion using representation matrices 
% This doc is going to show how to do data fusion using representation 
% matrices. 

%% Add current folder to the path
clear all;
current_fold = pwd;
addpath(genpath(current_fold));

%% The Simulation of coupled binary and quantitative data sets
% The simulation process can be referred to the accompanied software of the
% paper "Generalized Simultaneous Component Analysis of Binary and
% Quantitative data" in <https://arxiv.org/abs/1807.04982>. We use the 
% logit transform of the empirical marginal probabilities of the
% binary CNA data set as the simulated offset term to simulate imbalanced
% binary data $\mathbf{X}_1$. The number of samples, binary variables and 
% quantitative variables are $m=160$, $n_1=410$, $n_2=1000$ respectively. 
% The noise level $\sigma^2$ in simulating quantitative data set $\mathbf{X}_2$ 
% is 1. The simulated low rank is 10. The SNRs in generating $\mathbf{X}_1$
% and $\mathbf{X}_2$ are both 1.

% 
% import real binary CNA data set
load('X1_CNA.mat');
mu1_fixed = logit(mean(X1_CNA,1)); % imbalanced binary data simulation
%mu1_fixed = zeros(1,size(X1_CNA,2)); % balanced binary data simulation

% data simulation
SNRs = [1,1];   % SRNs in simulating X1 and X2
K    = 3;       % simulated low rank
link = 'logit'; % use logit link
%link = 'probit';% use logit link
seed = 1234;    % set seed to reproduce the example
[X1,X2,Theta_simu,mu_simu,Z_simu,E_simu] = GSCA_data_simulation(mu1_fixed,SNRs,K,link,seed);

% plot of the simulated offset term
figure
plot(mu_simu); title('simulated offset'); xlabel('variables');

%% Representation matrices of binary data set X1
% Represent each binary variable in X1 by a representation matrix, then
% combine all $n_1$ representation matrices together into a $m \times m \times n_1$ 
% three way array. Here we use the matricized form $m \times m*n_1$ of the
% three way array.

% An example of representaion matrix of binary data
xj = X1(:,5); % index out the 5th binary variable
Sj = representationBinVar(xj); % represent matrix of xj

% Representation matrices of binary data X1
[m, n1] = size(X1);
SBin = zeros(m, m*n1);
for j = 1:n1
    xj = X1(:,j);
    Sj = representationBinVar(xj);
    SBin(:,(j-1)*m+1:j*m) = Sj;
end

%% Representation matrices of quantitative data set X2

%
% An example of representation matrix of quantitative data 
xj = X2(:,5);
Sj = representationQuantiVar( xj );

% Representation matrice of quantitative data X2
[~,n2] = size(X2);
SQuanti = zeros(m,m*n2);
for j = 1:n2
    xj = X2(:,j);
    Sj = representationQuantiVar(xj);
    SQuanti(:,(j-1)*m+1:j*m) = Sj;
end

%% Using INDSCAL or INDORT model to analysis the three way array
S = [SBin, SQuanti]; % similarity matrices
%size(S)             % check the dimensions
R      = 3;     % number of PCs   
start  = 1;     % using 0 as initilization 
conv   = 1e-8;  % convergence criteria
method = 1;     % using alternating minimization algorithm

%[AHat, BHat, out] = INDS(S, R, conv, start, method); % INDSCAL model
[AHat, BHat, out] = INDORT(S, R, conv, start);       % INDORT model 

%% Convergence behavior of the algorithm
figure
plot(out.obj(2:end));
xlabel('iterations'); 
ylabel('object loss'); 
title('convergence behavior');

%% Variation explained ratios
% Variation explained of the model and PCs
[varExp_total, varExp_PCs] = representation_varExp(S,R,AHat,BHat);
varExp_total = varExp_total*100;
varExp_PCs  = varExp_PCs*100;

% Variation explained in binary X1 by the model
[varExp_X1, varExp_PCs_X1] = representation_varExp(SBin,R,AHat,BHat(1:n1,:));
varExp_X1 = varExp_X1*100;
varExp_PCs_X1 = varExp_PCs_X1*100;

% Variation explained in quantitative X2 by the model
[varExp_X2, varExp_PCs_X2] = representation_varExp(SQuanti,R,AHat,BHat(n1+1:end,:));
varExp_X2 = varExp_X2*100;
varExp_PCs_X2 = varExp_PCs_X2*100;

disp('total variation explained for the full data sets or each single data set:');
[varExp_total, varExp_X1, varExp_X2]

disp('variaiton explained for different PCs for the full data sets or each single data set:');
[varExp_PCs', varExp_PCs_X1', varExp_PCs_X2']

%% Score and loading plots
B1Hat = BHat(1:n1,:);     % loading matrix for Binary data X1
B2Hat = BHat(n1+1:end,:); % loading matrix for quantitative data X2

figure
subplot(1,3,1)
plot(AHat(:,1), AHat(:,2), 'o');
xlabel('PC1'); ylabel('PC2'); title('score plot');
%xlim([-0.2,0.2]); ylim([-0.2,0.2]);
subplot(1,3,2)
plot(B1Hat(:,1), B1Hat(:,2), 'or');
xlabel('PC1'); title('loading plot of X1');
%xlim([0, 26]); ylim([0, 26]);
subplot(1,3,3);
plot(B2Hat(:,1), B2Hat(:,2), 'o');
xlabel('PC1'); title('loading plot of X2');
%xlim([0, 150]); ylim([0, 150]);

